<?php
namespace BooklyServiceSchedule\Backend\Modules\Services\ProxyProviders;

use Bookly\Lib as BooklyLib;
use Bookly\Backend\Modules\Services\Proxy;
use BooklyServiceSchedule\Lib;
use Bookly\Backend\Modules\Staff\Forms\Widgets\TimeChoice;

/**
 * Class Shared
 * @package BooklyServiceSchedule\Backend\Modules\Services\ProxyProviders
 */
class Shared extends Proxy\Shared
{
    /**
     * @inheritdoc
     */
    public static function enqueueAssetsForServices()
    {
        self::enqueueStyles( array(
            'bookly' => array( 'frontend/resources/css/ladda.min.css', ),
        ) );

        self::enqueueScripts( array(
            'module' => array(
                'js/service_schedule.js' => array( 'jquery' ),
            ),
            'bookly' => array(
                'backend/resources/js/moment.min.js',
                'frontend/resources/js/spin.min.js'  => array( 'jquery' ),
                'frontend/resources/js/ladda.min.js' => array( 'jquery' ),
            )
        ) );

        wp_localize_script( 'bookly-service_schedule.js', 'ServiceScheduleL10n', array(
            'are_you_sure' => __( 'Are you sure?', 'bookly' ),
            'saved'        => __( 'Settings saved.', 'bookly' ),
        ) );
    }

    /**
     * @inheritdoc
     */
    public static function prepareAfterServiceList( $html )
    {
        $working_start_time = '08:00:00';
        $working_end_time   = '18:00:00';
        $choice_start = new TimeChoice( array( 'use_empty' => false, 'type' => 'break_from' ) );
        $choice_end   = new TimeChoice( array( 'use_empty' => false, 'type' => 'to' ) );

        return $html . self::renderTemplate( 'popover', compact( 'working_start_time', 'working_end_time', 'choice_start', 'choice_end' ), false );
    }

    /**
     * @inheritdoc
     */
    public static function serviceCreated( BooklyLib\Entities\Service $service )
    {
        $sch_day = new Lib\Entities\ServiceScheduleDay();
        $sch_day->setServiceId( $service->getId() );
        $week = array( 1 => 'sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday' );
        foreach ( $week as $day_index => $day ) {
            $item = clone $sch_day;
            $item->setDayIndex( $day_index )
                 ->setStartTime( get_option( 'bookly_bh_' . $day . '_start' ) ?: null )
                 ->setEndTime( get_option( 'bookly_bh_' . $day . '_end' ) ?: null )
                 ->save();
        }
    }

    /**
     * @inheritdoc
     */
    public static function updateService( array $alert, BooklyLib\Entities\Service $service, array $_post )
    {
        /** @var Lib\Entities\ServiceScheduleDay[] $days */
        $days = Lib\Entities\ServiceScheduleDay::query()->where( 'service_id', $service->getId() )->indexBy( 'day_index' )->find();
        foreach ( $_post['service_schedule']['start_time'] as $day_index => $value ) {
            if ( isset( $days[ $day_index ] ) ) {
                $days[ $day_index ]->setStartTime( $value == '' ? null : $value )
                    ->setEndTime( $_post['service_schedule']['end_time'][ $day_index ] )
                    ->save();
            } else {
                $day = new Lib\Entities\ServiceScheduleDay();
                $day->setServiceId( $service->getId() )
                    ->setDayIndex( $day_index )
                    ->setStartTime( $value == '' ? null : $value )
                    ->setEndTime( $_post['service_schedule']['end_time'][ $day_index ] )
                    ->save();
            }
        }

        return $alert;
    }

}