jQuery(function ($) {
    $(document.body).on('service.initForm', {},
        // Bind an event handler to the components for service panel.
        function (event, $panel, service_id) {
            $container = $panel.find('#bookly-services-schedule');
            $container.off().on('change', '.bookly-js-weekly-container .parent-range-start', function () {
                var $parent_range_start = $(this),
                    $range_row          = $parent_range_start.parents('.bookly-range-row');
                if ($parent_range_start.val() == '') {
                    $range_row.find('.bookly-hide-on-off').hide();
                } else {
                    $range_row.find('.bookly-hide-on-off').show();
                    rangeTools.hideInaccessibleEndTime($parent_range_start, $range_row.find('.parent-range-end'));
                }
            }).on('click', '.bookly-js-service-schedule-save', function (e) {
                // Listener for saving break.
                var $button              = $(this),
                    $popover_content     = $(this).closest('.popover-content'),
                    service_sch_break_id = $button.data('service_sch_break_id'),
                    data                 = {
                        action             : 'bookly_service_schedule_save_break',
                        csrf_token         : BooklyL10n.csrfToken,
                        service_id         : service_id,
                        service_schedule_id: service_sch_break_id,
                        day_index          : $button.data('day_index'),
                        start_time         : $popover_content.find('.popover-range-start').val(),
                        end_time           : $popover_content.find('.popover-range-end').val()
                    },
                    ladda                = rangeTools.ladda(this);
                e.preventDefault();
                $.ajax({
                    method  : 'POST',
                    url     : ajaxurl,
                    data    : data,
                    dataType: 'json',
                    success : function (response) {
                        if (response.success) {
                            if (service_sch_break_id) {
                                // Change text on button with new range value.
                                var $interval_button = $('button.bookly-js-service-schedule-toggle-popover.break-interval[data-service_sch_break_id="' + service_sch_break_id + '"]');
                                $interval_button.html(response.data.interval);
                            } else {
                                var $html = $.parseHTML(response.data.html);
                                initContentListeners($html);
                                $button.parents('.bookly-breaks-wrapper').find('.bookly-range-breaks-list').append($html);
                            }
                            $button.parents('.popover').popover('hide');
                        } else {
                            if (response.data && response.data.message) {
                                booklyAlert({error: [response.data.message]});
                            }
                        }
                    }
                }).always(function () {
                    ladda.stop()
                });
            }).on('change', '.popover-range-start', function () {
                var $start = $(this);
                var $end = $start.parents('.bookly-flexbox').find('.popover-range-end');
                rangeTools.hideInaccessibleBreaks($start, $end);
            }).on('click', '.delete-break[data-service_sch_break_id]', function (e) {
                // Delete break.
                e.preventDefault();
                if (confirm(ServiceScheduleL10n.are_you_sure)) {
                    var $button = $(this),
                        ladda   = rangeTools.ladda(this);
                    $.ajax({
                        method  : 'POST',
                        url     : ajaxurl,
                        data    : {action: 'bookly_service_schedule_delete_break', id: $button.data('service_sch_break_id'), csrf_token: BooklyL10n.csrfToken},
                        dataType: 'json',
                        success : function (response) {
                            if (response.success) {
                                $button.closest('.bookly-intervals-wrapper').remove();
                            } else {
                                if (response.data && response.data.message) {
                                    booklyAlert({error: [response.data.message]});
                                }
                            }
                        }
                    }).always(function () {
                        ladda.stop();
                    });
                }
            }).on('click', 'button.bookly-js-reset', function (e) {
                $container.find('form').trigger('reset');
            }).on('click', 'button.bookly-js-save-schedule', function (e) {
                e.preventDefault();
                var data  = $(this).closest('form').serializeArray(),
                    ladda = Ladda.create(this);
                ladda.start();

                data.push({name: 'action', value: 'bookly_service_schedule_update_schedule'});
                $.ajax({
                    type       : 'POST',
                    url        : ajaxurl,
                    data       : data,
                    dataType   : 'json',
                    xhrFields  : {withCredentials: true},
                    crossDomain: 'withCredentials' in new XMLHttpRequest(),
                    success    : function (response) {
                        ladda.stop();
                        booklyAlert(response.data.alert);
                    }
                });
            })
            ;
            $container.find('.parent-range-start').trigger('change');
            initContentListeners($container);

            function initContentListeners($panel) {
                $('.bookly-js-service-schedule-toggle-popover', $panel).popover({
                    html     : true,
                    placement: 'bottom',
                    template : '<div class="popover" role="tooltip"><div class="popover-arrow"></div><h3 class="popover-title"></h3><div class="popover-content"></div></div>',
                    trigger  : 'manual',
                    content  : function () {
                        return $('.bookly-js-service-schedule-popup-content').html();
                    }
                }).on('click', function () {
                    var $button = $(this);
                    $button.popover('toggle');
                    var $popover          = $button.next('.popover'),
                        $popover_content  = $popover.find('.popover-content'),
                        $popover_start    = $popover_content.find('.popover-range-start'),
                        $popover_end      = $popover_content.find('.popover-range-end'),
                        $save_button      = $popover_content.find('.bookly-js-service-schedule-save'),
                        force_keep_values = false;
                    $('.popover').not($popover).popover('hide');
                    if ($button.hasClass('break-interval')) {
                        var interval = $button.html().trim().split(' - ');
                        rangeTools.setVal($popover_start, interval[0]);
                        rangeTools.setVal($popover_end, interval[1]);
                        force_keep_values = true;
                        $save_button.data('service_sch_break_id', $button.data('service_sch_break_id')).data('ssd_id', $button.data('ssd_id')).data('day_index', $button.data('day_index'));
                    } else {
                        rangeTools.setPopoverRangeDefault($popover_start, $popover_end, $button.closest('.bookly-range-row'));
                        $save_button.data('day_index', $button.data('day_index'));
                    }
                    rangeTools.hideInaccessibleBreaks($popover_start, $popover_end, force_keep_values);
                    $popover.find('.bookly-popover-close').on('click', function () {
                        $popover.popover('hide');
                    });
                });
            }
        }
    )
});