<?php
namespace BooklySpecialDays\Backend\Components\Dialogs\Staff\Edit\ProxyProviders;

use Bookly\Backend\Modules\Staff;
use Bookly\Lib as BooklyLib;
use BooklySpecialDays\Lib;
use Bookly\Backend\Components\Dialogs\Staff\Edit\Proxy;

/**
 * Class Local
 *
 * @package BooklySpecialDays\Backend\Components\Dialogs\Staff\Edit\ProxyProviders
 */
class Local extends Proxy\SpecialDays
{
    /**
     * @inheritdoc
     */
    public static function getStaffSpecialDaysHtml( $default, $staff_id )
    {
        if ( ! BooklyLib\Utils\Common::isCurrentUserAdmin() ) {
            do {
                $staff = new BooklyLib\Entities\Staff();
                if ( $staff->load( $staff_id ) && $staff->getWpUserId() == get_current_user_id() ) {
                    break;
                }
                return 'Bookly: ' . __( 'You do not have sufficient permissions to access this page.' );
            } while ( 0 );
        }
        $special_days  = self::getSpecialDays( $staff_id );
        $working_end   = new Staff\Forms\Widgets\TimeChoice( array( 'use_empty' => false, 'type' => 'to' ) );
        $working_start = new Staff\Forms\Widgets\TimeChoice( array( 'use_empty' => false, 'type' => 'from' ) );

        return \BooklySpecialDays\Backend\Modules\Staff\Ajax::renderTemplate( 'list', compact( 'special_days', 'staff_id', 'working_start', 'working_end' ), false );
    }

    /******************************************************************************************************************
     * Protected methods                                                                                              *
     ******************************************************************************************************************/

    /**
     * Get special days items of staff member.
     *
     * @param int $staff_id
     * @return array [ id, date, start_time, end_time, breaks[ [ id, start_time, end_time, day_id ] ] ]
     */
    protected static function getSpecialDays( $staff_id )
    {
        $special_days = Lib\Entities\StaffSpecialDay::query( 'sd' )
            ->select( 'sd.id, sd.date, sd.start_time, sd.end_time' )
            ->where( 'sd.staff_id', $staff_id )
            ->indexBy( 'id' )
            ->sortBy( 'sd.date' )
            ->fetchArray();
        if ( is_array( $special_days ) ) {
            array_walk( $special_days, function ( &$day ) {
                $day['breaks'] = array();
            } );
            $breaks = Lib\Entities\StaffSpecialDayBreak::query( 'sb' )
                ->select( 'sb.id, sb.start_time, sb.end_time, sb.staff_special_day_id AS day_id' )
                ->whereIn( 'sb.staff_special_day_id', array_keys( $special_days ) )
                ->sortBy( 'sb.start_time' )
                ->fetchArray();
            foreach ( $breaks as $break ) {
                $special_days[ $break['day_id'] ]['breaks'][] = $break;
            }
            return $special_days;
        }

        return array();
    }
}