<?php
namespace BooklySpecialDays\Backend\Modules\Services\ProxyProviders;

use Bookly\Lib as BooklyLib;
use Bookly\Backend\Modules\Services\Proxy;
use BooklySpecialDays\Backend\Modules\Services\Forms;
use BooklySpecialDays\Lib;

/**
 * Class Shared
 * @package BooklySpecialDays\Backend\Modules\Services\ProxyProviders
 */
class Shared extends Proxy\Shared
{
    /**
     * @inheritdoc
     */
    public static function enqueueAssetsForServices()
    {
        self::enqueueStyles( array(
            'bookly' => array( 'frontend/resources/css/ladda.min.css', 'backend/resources/css/jquery-ui-theme/jquery-ui.min.css', ),
        ) );

        self::enqueueScripts( array(
            'module' => array( 'js/service_special_days.js' => array( 'jquery', 'jquery-ui-datepicker' ), ),
            'bookly' => array(
                'backend/resources/js/moment.min.js',
                'frontend/resources/js/spin.min.js'  => array( 'jquery' ),
                'frontend/resources/js/ladda.min.js' => array( 'jquery' ),
            )
        ) );

        wp_localize_script( 'bookly-service_special_days.js', 'ServiceSpecialDaysL10n', array(
            'datePicker' => BooklyLib\Utils\DateTime::datePickerOptions(),
            'are_you_sure' => __( 'Are you sure?', 'bookly' ),
            'start_time'   => '08:00:00',
            'end_time'     => '18:00:00',
            'saved'        => __( 'Settings saved.', 'bookly' ),
            'duplicate_error' => __( 'Duplicate dates are not permitted.', 'bookly'),
        ) );
    }

    /**
     * @inheritdoc
     */
    public static function updateService( array $alert, BooklyLib\Entities\Service $service, array $_post )
    {
        if ( isset( $_post['special_days'] ) ) {
            $duplicate_ids = array();
            foreach ( $_post['special_days'] as $data ) {
                $ssd = Lib\Entities\ServiceSpecialDay::query()
                    ->select( 'id' )
                    ->where( 'date', $data['date'] )
                    ->where( 'service_id', $service->getId() );

                if ( isset( $data['id'] ) ) {
                    $ssd->whereNot( 'id', $data['id'] );
                }
                $ssd = $ssd->fetchRow();
                if ( ! empty( $ssd ) ) {
                    $duplicate_ids[] = $data['id'];
                } else {
                    $form               = new Forms\ServiceSpecialDay();
                    $data['service_id'] = $service->getId();
                    $form->bind( $data );
                    $form->save();
                    BooklyLib\Entities\Stat::record( 'special_days_changed', BooklyLib\Base\Schema::getAffectedRows() );
                }
            }
            if ( $duplicate_ids ) {
                $alert['error'][] = __( 'Duplicate dates are not permitted.', 'bookly' );
            }
        }

        return $alert;
    }
}