jQuery(function ($) {
    $(document.body).on('service.initForm', {},
        // Bind an event handler to the components for service panel.
        function (event, $container, service_id) {
            initContentListeners($container);
            let special_days_popup_content = $container.find('.bookly-js-service-special-days-popup-content').html();
            $container.off().on('click', '.bookly-js-service-special-days-add-day', function () {
                // Listener for saving new special days.
                var data  = {
                        action    : 'bookly_special_days_add_for_service',
                        csrf_token: BooklyL10n.csrfToken,
                        service_id: service_id,
                        start_time: ServiceSpecialDaysL10n.start_time,
                        end_time  : ServiceSpecialDaysL10n.end_time
                    },
                    ladda = rangeTools.ladda(this);
                $.ajax({
                    method  : 'POST',
                    url     : ajaxurl,
                    data    : data,
                    dataType: 'json',
                    success : function (response) {
                        if (response.success) {
                            var $html = $.parseHTML(response.data.html);
                            initContentListeners($html);
                            $('.bookly-js-special-days-list', $container).append($html);
                            $('.bookly-js-special-days-list .bookly-js-date:last', $container).focus();
                        } else {
                            if (response.data && response.data.message) {
                                booklyAlert({error: [response.data.message]});
                            }
                        }
                    }
                }).always(function () {
                    ladda.stop();
                });
            }).on('click', '#bookly-js-service-special-days-save-days', function () {
                // Listener for saving new special days in modal window.
                var $form = $(this).closest('form'),
                    ladda = rangeTools.ladda(this);
                var data = {};

                /* Convert customer date format to YYYY-MM-DD */
                $form.serializeArray().map(function (elem) {
                    if (elem.name.indexOf('[date]') !== -1) {
                        data[elem.name] = moment($form.find('[name="' + elem.name + '"]').datepicker('getDate')).format('YYYY-MM-DD')
                    } else {
                        data[elem.name] = elem.value;
                    }
                });
                $.ajax({
                    method     : 'POST',
                    url        : ajaxurl,
                    data       : data,
                    dataType   : 'json',
                    xhrFields  : {withCredentials: true},
                    crossDomain: 'withCredentials' in new XMLHttpRequest(),
                    success    : function (response) {
                        var $ssd_container = $('.bookly-js-service-special-days-container', $container);
                        if (response.success) {
                            booklyAlert({success: [ServiceSpecialDaysL10n.saved]});
                            $ssd_container.find('.row.bg-danger').removeClass('bg-danger');
                        } else {
                            if (response.data) {
                                if (response.data.message) {
                                    booklyAlert({error: [response.data.message]});
                                    response.data.ids.forEach(function (ssd_row_id) {
                                        $ssd_container.find('.bookly-range-row.ssd_id-' + ssd_row_id).find('.row').addClass('bg-danger');
                                    });
                                }
                            }
                        }
                    }
                }).always(function () {
                    ladda.stop();
                });
            }).on('click', '.bookly-js-service-special-days-save-break', function (e) {
                // Listener for saving break.
                var $button        = $(this),
                    $sd_break_form = $(this).closest('.popover-content'),
                    ssd_break_id   = $button.data('ssd_break_id'),
                    data           = {
                        action                        : 'bookly_special_days_save_break_for_service',
                        csrf_token                    : BooklyL10n.csrfToken,
                        service_special_day_id        : $button.data('ssd_id'),
                        service_special_days_breaks_id: ssd_break_id,
                        start_time                    : $sd_break_form.find('.service-special-days-break-start').val(),
                        end_time                      : $sd_break_form.find('.service-special-days-break-end').val()
                    },
                    ladda          = rangeTools.ladda(this);
                e.preventDefault();
                $.ajax({
                    method  : 'POST',
                    url     : ajaxurl,
                    data    : data,
                    dataType: 'json',
                    success : function (response) {
                        if (response.success) {
                            if (ssd_break_id) {
                                // Change text on button with new range value.
                                var $interval_button = $('button.bookly-js-service-special-days-toggle-popover.break-interval[data-ssd_break_id="' + ssd_break_id + '"]');
                                $interval_button.html(response.data.interval);
                            } else {
                                var $html = $.parseHTML(response.data.html);
                                initContentListeners($html);
                                $button.closest('.bookly-js-service-special-days-breaks-list').find('.breaks-list-content').append($html);
                            }
                            booklyAlert({success: [ServiceSpecialDaysL10n.saved]});
                            $button.closest('.popover').popover('hide');
                        } else {
                            if (response.data && response.data.message) {
                                booklyAlert({error: [response.data.message]});
                            }
                        }
                    }
                }).always(function () {
                    ladda.stop()
                });
            }).on('change', '.service-special-days-break-start', function () {
                rangeTools.hideInaccessibleBreaks($(this), $(this).closest('.bookly-flexbox').find('.service-special-days-break-end'));
            }).on('click', '#bookly-js-service-special-days-delete', function (e) {
                e.preventDefault();
                if (confirm(ServiceSpecialDaysL10n.are_you_sure)) {
                    var ids = [];
                    var ssd_rows = [];
                    $(':checkbox[data-ssd_id]:checked').each(function (index, elem) {
                        ids.push($(elem).data('ssd_id'));
                        ssd_rows.push($(elem).closest('.bookly-range-row'));
                    });
                    if (ids.length > 0) {
                        var ladda = rangeTools.ladda(this);
                        $.ajax({
                            method     : 'POST',
                            url        : ajaxurl,
                            data       : {action: 'bookly_special_days_delete_days_for_service', ids: ids, service_id: service_id, csrf_token: BooklyL10n.csrfToken},
                            dataType   : 'json',
                            xhrFields  : {withCredentials: true},
                            crossDomain: 'withCredentials' in new XMLHttpRequest(),
                            success    : function (response) {
                                if (response.success) {
                                    ssd_rows.forEach(function (elem) {
                                        $(elem).remove();
                                    });
                                } else {
                                    if (response.data && response.data.message) {
                                        booklyAlert({error: [response.data.message]});
                                    }
                                }
                                findDuplicates($container);
                            }
                        }).always(function () {
                            ladda.stop()
                        });
                    }
                }
            }).on('click', '.delete-break[data-ssd_break_id]', function (e) {
                // Delete break.
                e.preventDefault();
                if (confirm(ServiceSpecialDaysL10n.are_you_sure)) {
                    var $button = $(this),
                        ladda   = rangeTools.ladda(this);
                    $.ajax({
                        method  : 'POST',
                        url     : ajaxurl,
                        data    : {action: 'bookly_special_days_delete_break_for_service', id: $button.data('ssd_break_id'), service_id: service_id, csrf_token: BooklyL10n.csrfToken},
                        dataType: 'json',
                        success : function (response) {
                            if (response.success) {
                                $button.closest('.bookly-breaks-wrapper').remove();
                            } else {
                                if (response.data && response.data.message) {
                                    booklyAlert({error: [response.data.message]});
                                }
                            }
                        }
                    }).always(function () {
                        ladda.stop();
                    });
                }
            }).on('change', '.bookly-js-date', function () {
                findDuplicates($container, this);
            });

            function initContentListeners($content) {
                $('.bookly-js-date', $content).datepicker(ServiceSpecialDaysL10n.datePicker);

                $('.parent-range-start', $content).each(function (index, elem) {
                    rangeTools.hideUp24Hours($(elem), $(elem).closest('.bookly-range-row').find('.parent-range-end'));
                });
                // init 'add break' functionality fos special days
                $('.bookly-js-service-special-days-toggle-popover', $content).popover({
                    html     : true,
                    placement: 'bottom',
                    template : '<div class="popover" role="tooltip"><div class="popover-arrow"></div><h3 class="popover-title"></h3><div class="popover-content"></div></div>',
                    trigger  : 'manual',
                    content  : function () {
                        return special_days_popup_content;
                    }
                }).on('click', function () {
                    var $button = $(this);
                    $button.popover('toggle');
                    var $popover          = $button.next('.popover'),
                        $form             = $popover.find('.popover-content'),
                        $break_start      = $form.find('.service-special-days-break-start'),
                        $break_end        = $form.find('.service-special-days-break-end'),
                        force_keep_values = false;
                    $('.popover').not($popover).popover('hide');
                    if ($button.hasClass('break-interval')) {
                        var interval = $button.html().trim().split(' - ');
                        rangeTools.setVal($break_start, interval[0]);
                        rangeTools.setVal($break_end, interval[1]);
                        force_keep_values = true;
                        $form.find('.bookly-js-service-special-days-save-break').data('ssd_break_id', $button.data('ssd_break_id')).data('ssd_id', $button.data('ssd_id'));
                    } else {
                        var $ssd_row = $button.closest('.row');
                        $break_start.val($ssd_row.find('.parent-range-start').val());
                        $break_end.val($ssd_row.find('.parent-range-end').val());
                        $form.find('.bookly-js-service-special-days-save-break').data('ssd_id', $button.data('ssd_id')).removeAttr('ssd_break_id');
                        rangeTools.setPopoverRangeDefault($break_start, $break_end, $button.closest('.bookly-range-row'));
                    }
                    rangeTools.hideInaccessibleBreaks($break_start, $break_end, force_keep_values);
                    $popover.find('.bookly-popover-close').on('click', function () {
                        $popover.popover('hide');
                    });
                });
            }

            /**
             * Function to find and mark duplicated dates
             * The same function presented in service_days.js
             *
             * @param $container
             * @param that          field that was changed, undefined if it was another action, like deletion field or reset form
             */
            function findDuplicates($container, that) {
                var all_dates        = [],
                    found_duplicates = false,
                    $saveButton      = $('#bookly-save', $container.closest('.modal-content')),
                    $serviceError    = $('.bookly-js-service-error', $container.closest('.modal-content'))
                ;
                /**
                 * Fill additional field that will be passed on backend
                 */
                if (that) {
                    var new_date = moment($(that).datepicker('getDate')).format('YYYY-MM-DD');
                    $container.find('[name="special_days[' + $(that).data('day_id') + '][date]"]')
                        .val(new_date);
                }

                $container.find('.bookly-range-row .bookly-js-date').each(function () {
                    all_dates.push($(this).val());
                });

                $container.find('.bookly-range-row').each(function () {
                    $(this).find('.row').removeClass('bg-danger');

                    var current_row_date = $(this).find('.bookly-js-date').val();
                    var duplicated_dates = all_dates.filter(function (item) {
                        return item === current_row_date;
                    });

                    if (duplicated_dates.length > 1 && current_row_date) {
                        $(this).find('.row').addClass('bg-danger');
                        found_duplicates = true;
                    }
                });

                if (found_duplicates) {
                    $serviceError.find('.bookly-js-special-days-error').remove();
                    $serviceError.append('<div class="bookly-js-special-days-error bookly-js-error">' + ServiceSpecialDaysL10n.duplicate_error + '</div>');

                } else {
                    $serviceError.find('.bookly-js-special-days-error').remove();
                }
                $saveButton.prop('disabled', $serviceError.find('.bookly-js-error').length > 0);
            }
        }
    )
});