jQuery(function ($) {

    var SpecialDays = function($container,options) {
        var obj = this,
            special_days_popup_content;
        jQuery.extend(obj.options, options);

        if ($('>', $container).length == 0) {
            $container.html('<div class="bookly-loading"></div>');
            $.ajax({
                type: 'POST',
                url: ajaxurl,
                data: {action: 'bookly_special_days_staff_special_days', staff_id: obj.options.staff_id, csrf_token: obj.options.csrf_token},
                dataType: 'json',
                xhrFields: {withCredentials: true},
                crossDomain: 'withCredentials' in new XMLHttpRequest(),
                success: function (response) {
                    var $html = $.parseHTML(response.data.html);
                    initContentListeners($html);
                    // fill in the container
                    $container.html($html);
                    init($container, obj);
                }
            });
        } else {
            initContentListeners($container);
            init($container, obj);
        }

        function init($container, obj) {
            if ($container.data('init') != true) {
                    special_days_popup_content = $container.find('.bookly-js-special-days-popup-content').html();

                    // Set listeners for special days container.
                    $container.on('click', '#bookly-js-special-days-add-day', function () {
                        // Listener for saving new special days.
                        var ladda = rangeTools.ladda(this);
                        $.ajax({
                            method: 'POST',
                            url: ajaxurl,
                            data: {
                                action: 'bookly_special_days_add_day',
                                csrf_token: obj.options.csrf_token,
                                staff_id: obj.options.staff_id,
                                start_time: obj.options.l10n.start_time,
                                end_time: obj.options.l10n.end_time
                            },
                            dataType: 'json',
                            xhrFields: {withCredentials: true},
                            crossDomain: 'withCredentials' in new XMLHttpRequest(),
                            success: function (response) {
                                if (response.success) {
                                    var $html = $.parseHTML(response.data.html);
                                    initContentListeners($html);
                                    $('input[name="action"]', $container).before($html);
                                    $('.bookly-js-date:last', $container).focus();
                                } else {
                                    if (response.data && response.data.message) {
                                        obj.options.savingStatus({error: [response.data.message]});
                                    }
                                }
                            }
                        }).always(function () {
                            ladda.stop();
                        });
                    }).on('click', '#bookly-js-special-days-reset', function (e) {
                        $(this).closest('form').trigger('reset');
                        findDuplicates();
                        e.preventDefault();
                    }).on('change', '.bookly-js-date', function () {
                        findDuplicates();
                    }).on('click', '#bookly-js-special-days-save-days', function () {
                        // Listener for saving new special days in modal window.
                        var $form = $(this).closest('form'),
                            ladda = rangeTools.ladda(this),
                            data = {};

                        /* Convert customer date format to YYYY-MM-DD */
                        $form.serializeArray().map(function (elem) {
                            if (elem.name.indexOf('[date]') !== -1) {
                                var date = moment($form.find('[name="' + elem.name + '"]').datepicker('getDate')).format('YYYY-MM-DD');
                                if (date != 'Invalid date') {
                                    data[elem.name] = date;
                                }
                            } else {
                                data[elem.name] = elem.value;
                            }
                        });
                        $.ajax({
                            method: 'POST',
                            url: ajaxurl,
                            data: data,
                            dataType: 'json',
                            xhrFields: {withCredentials: true},
                            crossDomain: 'withCredentials' in new XMLHttpRequest(),
                            success: function (response) {
                                if (response.success) {
                                    obj.options.savingStatus({success: [obj.options.l10n.saved]});
                                    $container.find('.row.bg-danger').removeClass('bg-danger');
                                } else {
                                    if (response.data) {
                                        if (response.data.message) {
                                            obj.options.savingStatus({error: [response.data.message]});
                                            response.data.ids.forEach(function (ssd_row_id) {
                                                $container.find('.bookly-range-row.ssd_id-' + ssd_row_id).find('.row').addClass('bg-danger');
                                            });
                                        }
                                    }
                                }
                            }
                        }).always(function () {
                            ladda.stop();
                        });
                    }).on('click', '.bookly-js-special-days-save-break', function (e) {
                        // Listener for saving break.
                        var $button = $(this),
                            $sd_break_form = $(this).closest('.popover-content'),
                            ssd_break_id = $button.data('ssd_break_id'),
                            ladda = rangeTools.ladda(this);
                        e.preventDefault();
                        $.ajax({
                            method: 'POST',
                            url: ajaxurl,
                            data: {
                                action: 'bookly_special_days_save_break',
                                csrf_token: obj.options.csrf_token,
                                staff_special_day_id: $button.data('ssd_id'),
                                staff_special_days_breaks_id: ssd_break_id,
                                start_time: $sd_break_form.find('.special-days-break-start').val(),
                                end_time: $sd_break_form.find('.special-days-break-end').val()
                            },
                            dataType: 'json',
                            xhrFields: {withCredentials: true},
                            crossDomain: 'withCredentials' in new XMLHttpRequest(),
                            success: function (response) {
                                if (response.success) {
                                    if (ssd_break_id) {
                                        // Change text on button with new range value.
                                        var $interval_button = $('button.bookly-js-special-days-toggle-popover.break-interval[data-ssd_break_id="' + ssd_break_id + '"]');
                                        $interval_button.html(response.data.interval);
                                    } else {
                                        var $html = $.parseHTML(response.data.html);
                                        initContentListeners($html);
                                        $button.parents('.bookly-js-special-days-breaks-list').find('.breaks-list-content').append($html);
                                    }
                                    $button.parents('.popover').popover('hide');
                                } else {
                                    if (response.data && response.data.message) {
                                        obj.options.booklyAlert({error: [response.data.message]});;
                                    }
                                }
                            }
                        }).always(function () {
                            ladda.stop()
                        });
                    }).on('change', '.special-days-break-start', function () {
                        rangeTools.hideInaccessibleBreaks($(this), $(this).parents('.bookly-flexbox').find('.special-days-break-end'));
                    }).on('click', '#bookly-js-special-days-delete-all', function (e) {
                        e.preventDefault();
                        if (confirm(obj.options.l10n.are_you_sure)) {
                            var ids = [];
                            var ssd_rows = [];
                            $(':checkbox[data-ssd_id]:checked').each(function (index, elem) {
                                ids.push($(elem).data('ssd_id'));
                                ssd_rows.push($(elem).closest('.bookly-range-row'));
                            });
                            if (ids.length > 0) {
                                var ladda = rangeTools.ladda(this);
                                $.ajax({
                                    method: 'POST',
                                    url: ajaxurl,
                                    data: {action: 'bookly_special_days_delete_days', ids: ids, staff_id: obj.options.staff_id, csrf_token: obj.options.csrf_token},
                                    dataType: 'json',
                                    xhrFields: {withCredentials: true},
                                    crossDomain: 'withCredentials' in new XMLHttpRequest(),
                                    success: function (response) {
                                        if (response.success) {
                                            ssd_rows.forEach(function (elem) {
                                                $(elem).remove();
                                            });
                                        } else {
                                            if (response.data && response.data.message) {
                                                obj.options.savingStatus({error: [response.data.message]});
                                            }
                                        }
                                        findDuplicates();
                                    }
                                }).always(function () {
                                    ladda.stop()
                                });
                            }
                        }
                    }).on('click', '.delete-break[data-ssd_break_id]', function (e) {
                        // Delete break.
                        e.preventDefault();
                        if (confirm(obj.options.l10n.are_you_sure)) {
                            var $button = $(this),
                                ladda = rangeTools.ladda(this);
                            $.ajax({
                                method: 'POST',
                                url: ajaxurl,
                                data: {action: 'bookly_special_days_delete_break', id: $button.data('ssd_break_id'), staff_id: obj.options.staff_id, csrf_token: obj.options.csrf_token},
                                dataType: 'json',
                                xhrFields: {withCredentials: true},
                                crossDomain: 'withCredentials' in new XMLHttpRequest(),
                                success: function (response) {
                                    if (response.success) {
                                        $button.closest('.bookly-breaks-wrapper').remove();
                                    } else {
                                        if (response.data && response.data.message) {
                                            obj.options.savingStatus({error: [response.data.message]});
                                        }
                                    }
                                }
                            }).always(function () {
                                ladda.stop();
                            });
                        }
                    }); // end listeners for special days container.
            }
            $container.data('init', true);
        }

        function initContentListeners ($content) {
            $('.bookly-js-date', $content).datepicker($.extend({
                beforeShow: function (input, inst) {
                    inst.dpDiv.queue(function () {
                        inst.dpDiv.css({'font-size': '13.5px'});
                        inst.dpDiv.dequeue();
                    });
                }
            }, obj.options.l10n.datePicker));

            $('.parent-range-start', $content).each(function (index, elem) {
                rangeTools.hideUp24Hours($(elem), $(elem).closest('.bookly-range-row').find('.parent-range-end'));
            });
            // init 'add break' functionality for special days
            $('.bookly-js-special-days-toggle-popover', $content).popover({
                html: true,
                placement: 'bottom',
                template: '<div class="popover" role="tooltip"><div class="popover-arrow"></div><h3 class="popover-title"></h3><div class="popover-content"></div></div>',
                trigger: 'manual',
                content: function () {
                    return special_days_popup_content;
                }
            }).on('click', function () {
                var $button = $(this);
                $button.popover('toggle');
                var $popover = $button.next('.popover'),
                    $form = $popover.find('.popover-content'),
                    $break_start = $form.find('.special-days-break-start'),
                    $break_end = $form.find('.special-days-break-end'),
                    force_keep_values = false;
                $('.popover').not($popover).popover('hide');
                if ($button.hasClass('break-interval')) {
                    var interval = $button.html().trim().split(' - ');
                    rangeTools.setVal($break_start, interval[0]);
                    rangeTools.setVal($break_end, interval[1]);
                    force_keep_values = true;
                    $form.find('.bookly-js-special-days-save-break').data('ssd_break_id', $button.data('ssd_break_id')).data('ssd_id', $button.data('ssd_id'));
                } else {
                    var $ssd_row = $button.closest('.row');
                    $break_start.val($ssd_row.find('.parent-range-start').val());
                    $break_end.val($ssd_row.find('.parent-range-end').val());
                    $form.find('.bookly-js-special-days-save-break').data('ssd_id', $button.data('ssd_id')).removeAttr('ssd_break_id');
                    rangeTools.setPopoverRangeDefault($break_start, $break_end, $button.closest('.bookly-range-row'));
                }
                rangeTools.hideInaccessibleBreaks($break_start, $break_end, force_keep_values);
                $popover.find('.bookly-popover-close').on('click', function () {
                    $popover.popover('hide');
                });
            });
        }

        /**
         * Function to find and mark duplicated dates
         * The same function presented in service_special_days.js
         */
        function findDuplicates()
        {
            var all_dates = [];
            $('.bookly-range-row .bookly-js-date', $container).each(function () {
                all_dates.push($(this).val());
            });

            var found_duplicates = false;

            $('.bookly-range-row',$container).each(function() {
                $(this).find('.row').removeClass('bg-danger');

                var current_row_date = $(this).find('.bookly-js-date').val();
                var duplicated_dates = all_dates.filter(function(item){
                    return item === current_row_date;
                });

                if (duplicated_dates.length > 1 && current_row_date) {
                    $(this).find('.row').addClass('bg-danger');
                    found_duplicates = true;
                }
            });

            if (found_duplicates) {
                $('.bookly-special-days-error').html(obj.options.l10n.duplicate_error);
                $('#bookly-js-special-days-save-days').prop('disabled', true);
            } else {
                $('.bookly-special-days-error').html('');
                $('#bookly-js-special-days-save-days').prop('disabled', false);
            }

            obj.options.validation(found_duplicates, obj.options.l10n.duplicate_error);
        }
    };

    SpecialDays.prototype.options = {
        id          : -1,
        csrf_token  : '',
        savingStatus: function (data) {
            $(document.body).trigger('staff_edit.save', [data]);
        },
        booklyAlert: function (data) {
            booklyAlert(data);
        },
        l10n        : {},
        validation: function (has_error, info) {
            $(document.body).trigger('staff_edit.validation', ['staff-special-days', has_error, info]);
        },
    };

    window.BooklyStaffSpecialDays = SpecialDays;
});
