<?php
namespace BooklyStaffCabinet\Frontend\Modules\StaffSchedule;

use Bookly\Backend\Modules\Staff\Forms\Widgets\TimeChoice;
use Bookly\Lib as BooklyLib;

/**
 * Class Ajax
 * @package BooklyStaffCabinet\Frontend\Modules\StaffSchedule
 */
class Ajax extends BooklyLib\Base\Ajax
{
    /**
     * @inheritdoc
     */
    protected static function permissions()
    {
        return array(
            'getSchedule' => 'user',
            'resetBreaks' => 'user',
        );
    }

    public static function getSchedule()
    {
        $staff    = BooklyLib\Entities\Staff::query()->where( 'wp_user_id', get_current_user_id() )->findOne();
        $staff_id = $staff->getId();
        $schedule_items = $staff->getScheduleItems();
        $html  = self::renderTemplate( 'schedule', compact( 'schedule_items', 'staff_id' ), false );
        wp_send_json_success( compact( 'html' ) );
    }

    /**
     * Reset breaks.
     */
    public static function resetBreaks()
    {
        $breaks = self::parameter( 'breaks' );
        $staff  = new BooklyLib\Entities\Staff();
        $staff->load( $breaks['staff_id'] );

        // Check permissions to prevent one staff member from updating profile of another staff member.
        if ( ! BooklyLib\Utils\Common::isCurrentUserAdmin() && $staff->getWpUserId() != get_current_user_id() ) {
            do_action( 'admin_page_access_denied' );
            wp_die( 'Bookly: ' . __( 'You do not have sufficient permissions to access this page.' ) );
        }

        $html_breaks = array();

        // Remove all breaks for staff member.
        $break = new BooklyLib\Entities\ScheduleItemBreak();
        $break->removeBreaksByStaffId( $breaks['staff_id'] );

        // Restore previous breaks.
        if ( isset( $breaks['breaks'] ) && is_array( $breaks['breaks'] ) ) {
            foreach ( $breaks['breaks'] as $day ) {
                $schedule_item_break = new BooklyLib\Entities\ScheduleItemBreak();
                $schedule_item_break->setFields( $day );
                $schedule_item_break->save();
            }
        }

        // Make array with breaks (html) for each day.
        foreach ( $staff->getScheduleItems() as $item ) {
            /** @var BooklyLib\Entities\StaffScheduleItem $item */
            $html_breaks[ $item->getId() ] = self::renderTemplate( '_breaks', array(
                'day_is_not_available' => null === $item->getStartTime(),
                'item'                 => $item,
                'break_start'          => new TimeChoice( array( 'use_empty' => false, 'type' => 'break_from' ) ),
                'break_end'            => new TimeChoice( array( 'use_empty' => false, 'type' => 'to' ) ),
            ), false );
        }

        wp_send_json( $html_breaks );
    }
}